#-----------------------------------------------------------------------------------#
#                                                                                   #
#                           PROTIS 2021-2022                                        #
#                                                                                   #
#-----------------------------------------------------------------------------------#
#                           Etienne LOISELET                                        #
#-----------------------------------------------------------------------------------#

# Institut d'Optique Graduate School (https://www.institutoptique.fr/)
# LEnsE (http://lense.institutoptique.fr/)

# Au 2A qui veut utiliser ce programme et ses librairies: en cas de question(s) ne pas
# hésiter à me contacter par mail à etienne.loiselet@institutoptique.fr ou sur messenger

## Importations:

import PathFinding          # Librairie spécifique au projet (gère le calcul du meilleur chemin)
import ConnectionNucleo     # Librairie spécifique au projet (gère la connexion avec la Nucléo)
import TraductionChemin     # Librairie spécifique au projet (gère la traduction des chemins entre les repères)

import PIL.Image        # Librairie générale de python (interfaçage)
import PIL.ImageTk      # Librairie générale de python (interfaçage)
import math as m        # Librairie générale de python (interfaçage)
import threading        # Librairie générale de python (interfaçage)
import tkinter as tk    # Librairie générale de python (interfaçage)
import time             # Librairie générale de python (interfaçage)
import importlib        # Librairie générale de python (permet de mettre à jour une librairie)

importlib.reload(ConnectionNucleo)  # mise à jour de "ConnectionNucleo" et "TraductionChemin" pour appliquer
importlib.reload(TraductionChemin)  # les potentielles modifications sans avoir à tout redémarrer

## Classes utilisées:

class Case_Legende(tk.Canvas):
    """ Classe définie pour des raisons pratiques. Permet de définir facilement une case de la carte comme item dans la légende.
    Utilisation: case=Case_Legende(parent,couleur)
    parent: est l'adresse de l'objet auquel la case sera liée
    couleur: est une chaine de caractère indiquant la couleur désirée de la case"""
    def __init__(self,parent,color):
        taille=17
        tk.Canvas.__init__(self,parent,bg='blue',width=taille,height=taille,bd=0)
        self.case=self.create_rectangle(2,2,taille+1,taille+1,fill=color,width=1,outline='black')

class Canvas_Perso(tk.Canvas):
    """ Classe générant un objet de type Canvas (hérité de Tkinter), c'est une zone de dessin sur laquelle
    sera représentée la carte du hangar.
    Pour créer une zone de dessin, exécuter la commande suivante:
    XX=Canvas_perso(fenetre ou objet parent, hauteur de la zone, nombre de case sur une ligne, sur une colonne)
    la largeur de la zone est automatique calculée puisque les cases doivent être des carrés"""
    def __init__(self,appli,parent,hauteur,t,n_c,n_l):

        ratio=n_c/n_l
        couleur="blue"          # couleur de survol des cases
        b=2                     # taille du bord d'une case
        tk.Canvas.__init__(self,parent,height=hauteur+1,width=round(ratio*hauteur)+1,bg='blue')

        self.liste_cases=[0]*n_l                # Liste qui contiendra toutes les cases de la carte
        for ligne in range(n_l):                # Elle est entièrement initialisée à 0
            self.liste_cases[ligne]=[0]*n_c

        for l in range(n_l):                    # Remplissage des cases conformément à la carte du hangar définie dans l'application
            for c in range(n_c):
                self.liste_cases[l][c]=self.create_rectangle(c*t+b,l*t+b,(c+1)*t+b,(l+1)*t+b,fill='white',width=1,outline='black',activefill=couleur)     # On remplit d'abord TOUTES les cases en blanc
                self.tag_bind(self.liste_cases[l][c],'<Button-1>',lambda evt: clic_case(appli,evt)) # On y attache une fonction qui réagit quand la case est cliquée
                if appli.mapbase[l,c]==1:           # Si la case était censée être un mur on la recolorie en noir
                    self.itemconfigure(self.liste_cases[l][c],fill='black')

        # On colorie les case de départ, arrivée et station définies par défaut
        self.itemconfigure(self.liste_cases[appli.case_depart[0]][appli.case_depart[1]],fill='green')
        self.itemconfigure(self.liste_cases[appli.case_arrivee[0]][appli.case_arrivee[1]],fill='red')
        self.itemconfigure(self.liste_cases[appli.case_station[0]][appli.case_station[1]],fill='cyan')

        # Option pour afficher le robot sur la carte (incomplet)
        # self.l_robot=2
        # self.c_robot=2
        # self.robot=self.create_oval(self.coords(self.liste_cases[self.l_robot][self.c_robot]),fill='red')

## Fonctions utilisées:


## Gestion du log:

def effacer_log(obj):
    """Fonction permettant d'effacer le contenu du log d'un "Logiciel" de la librairie "Application" et d'afficher un message de bienvenue
    Utilisation: effacer_log(adresse de l'objet "Logiciel" concerné)"""
    obj.log.config(state='normal')                          # On autorise la modification du log
    obj.log.delete("1.0","end")                             # On efface du début à la fin
    obj.log.insert('end','>>> Début du log de commandes')   # On affiche un message de bienvenue
    obj.log.config(state='disabled')                        # On empêche l'utilisateur de modifier le log

def ecrire_log(obj,texte):
    """ Fonction permettant d'écrire dans le log d'un "Logiciel" de la librairie "Application"
    Utilisation: ecrire_log("Je veux écrire ce texte")
    """
    obj.log.config(state='normal')      # On autorise la modification du log
    obj.log.insert('end', '\n'+texte)   # On écrit le texte voulu
    obj.log.config(state='disabled')    # On empêche l'utilisateur de modifier le log
    obj.log.see('end')                  # On fait en sorte que l'on voit la dernière ligne ajoutée

## Gestion des cases de la carte:

def reinitialise_couleur(obj,coord_case):
    """ Fonction permettant de réinitialiser la couleur d'une case conformément au fond de carte de l'application
    (noir ou blanc)
    Entrée: objet application et coordonnées de la case à réinitialiser (tuple ou liste de taille 1x2)"""
    if obj.mapbase[coord_case]==0:      # Si la case doit être blanche on la colorie en blanc
        obj.carte.itemconfigure(obj.carte.liste_cases[coord_case[0]][coord_case[1]],fill='white')
    elif obj.mapbase[coord_case]==1:    # Si la case doit être noire on la colorie en noir
        obj.carte.itemconfigure(obj.carte.liste_cases[coord_case[0]][coord_case[1]],fill='black')
    else:
        ecrire_log(obj,"/!\ La valeur initiale de cette case n'est pas 0 ou 1 /!\ ")

def clic_case(obj,info_clic):
    """ Fonction appelée lorsque l'on clic sur une case, elle renvoie au log les coordonnées de la case sélectionnée
    et signale au reste de l'application qu'une case a été cliquée
    Entrée: objet application et informations du clic de la souris (passées automatiquement dans Application)"""
    obj.case_actu=(m.floor(info_clic.y/678*15),m.floor(info_clic.x/453*10))     # Enregistre dans l'application les coordonnées de la case (écrase la précédente)
    obj.event.set()     # Met à True l'évenement "Une case a été seléctionnée"
    ecrire_log(obj,">>> Vous avez cliqué sur la case: ("+str(obj.case_actu[0])+","+str(obj.case_actu[1])+")")

def def_depart(obj):
    """ Fonction interne appelée par le bouton : obj.bdepart. Elle a pour but de permettre à l'utilisateur de définir un nouveau
    point de départ pour le calcul du meilleur chemin.
    Effet: Une fois que l'utilisateur a cliqué sur le bouton, il doit cliquer sur une case de la carte. Celle-ci deviendra le
    nouveau départ et remplacera l'ancien. Si l'utilisateur met trop de temps à cliquer sur une case, la fonction s'arrête et
    l'utilisateur en est informé.
    Entrée: objet application """
    obj.event.clear()   # Met à False l'évenement "Une case a été seléctionnée"
    ecrire_log(obj,">>> Veuillez sélectionner une case de départ")
    thread=threading.Thread(target=interruption_depart,args=(obj,obj.event,)) # Création une fonction d'interruption attendant de l'utilisateur qu'il clique sur une case de la carte du hangar
    thread.start()  # Démarrage de la fonction d'interruption

def interruption_depart(obj,evt):
    """ Fonction d'interruption liée au bouton départ
    Elle redéfinit une nouvelle case de départ, la colorie et supprime l'ancien départ
    Entrée: objet application, évenement à prendre en compte pour l'interruption"""
    if evt.wait(3): # On regarde si l'évenement est True (l'utilisateur a cliqué sur une case) avant le temps imparti (3 secondes)
        ecrire_log(obj,">>> Nouveau départ défini") # On écrit sur le log que ça a marché
        reinitialise_couleur(obj,obj.case_depart)   # On réinitialise l'ancienne case de départ
        obj.case_depart=obj.case_actu               # On définit la nouvelle case de départ
        obj.carte.itemconfigure(obj.carte.liste_cases[obj.case_depart[0]][obj.case_depart[1]],fill='green') # On colorie le nouveau départ en vert
    else:
        ecrire_log(obj,">>> Vous avez mis trop de temps") # L'opération a échoué on le signale à l'utilisateur
    evt.clear()     # Met à False l'évenement "Une case a été sélectionnée"

def def_arrivee(obj):
    """ Voir la fonction def_depart, meme utilisation mais pour l'arrivée """
    obj.event.clear()
    ecrire_log(obj,">>> Veuillez sélectionner une case d'arrivée")
    thread=threading.Thread(target=interruption_arrivee,args=(obj,obj.event,))
    thread.start()

def interruption_arrivee(obj,evt):
    """ Voir la fonction interruption_depart, meme utilisation mais pour l'arrivée """
    if evt.wait(3):
        ecrire_log(obj,">>> Nouvelle arrivée définie")
        reinitialise_couleur(obj,obj.case_arrivee)
        obj.case_arrivee=obj.case_actu
        obj.carte.itemconfigure(obj.carte.liste_cases[obj.case_arrivee[0]][obj.case_arrivee[1]],fill='red')
    else:
        ecrire_log(obj,">>> Vous avez mis trop de temps")
    evt.clear()

def def_station(obj):
    """ Voir la fonction def_depart, meme utilisation mais pour la station """
    obj.event.clear()
    ecrire_log(obj,'>>> Veuillez sélectionner une case "station"')
    thread=threading.Thread(target=interruption_station,args=(obj,obj.event,))
    thread.start()

def interruption_station(obj,evt):
    """ Voir la fonction interruption_depart, meme utilisation mais pour la station """
    if evt.wait(3):
        ecrire_log(obj,">>> Nouvelle station définie")
        reinitialise_couleur(obj,obj.case_station)
        obj.case_station=obj.case_actu
        obj.carte.itemconfigure(obj.carte.liste_cases[obj.case_station[0]][obj.case_station[1]],fill='cyan')
    else:
        ecrire_log(obj,">>> Vous aves mis trop de temps")
    evt.clear()

def reinit_case(obj):
    """ Voir la fonction def_depart, meme utilisation mais pour réinitialiser une case """
    obj.event.clear()
    ecrire_log(obj,'>>> Veuillez sélectionner la case à réinitialiser')
    thread=threading.Thread(target=interruption_reinit,args=(obj,obj.event,))
    thread.start()

def interruption_reinit(obj,evt):
    """ Voir la fonction interruption_depart, meme utilisation mais pour réinitialiser une case """
    if evt.wait(3):
        ecrire_log(obj,">>> Case réinitialisée")
        reinitialise_couleur(obj,obj.case_actu)
    else:
        ecrire_log(obj,">>> Vous avez mis trop de temps")
    evt.clear()

def effacer_chemin(obj):
    """ Fonction qui réinitialise la couleur des cases du chemin"""
    for coord_case in obj.chemin[1:-1]:
        reinitialise_couleur(obj,coord_case)
    ecrire_log(obj,">>> Chemin effacé")

## Gestion de la connexion avec la Nucléo:

def liste_ports(objappli,objserie):
    """ Fonction établissant la liste des ports disponibles et mettant à jour la combobox destinée à les afficher"""
    objappli.ports = objserie.serial_ports()
    if(objappli.ports != 0):
        for i in range(0, len(objappli.ports)):
            objappli.cbx_listeports.insert(i, objappli.ports[i])
    else:
        objappli.cbx_listeports.insert(-1, 'No Serial')

def trouver_ports(objappli,objserie):
    """ Fonction cherchant les ports disponibles """
    liste_ports(objappli,objserie)
    ecrire_log(objappli,'>>> Détection des ports exploitables')
    if objappli.ports == 0:
        objappli.b_Connecter['state'] = 'disabled'
        ecrire_log(objappli,'>>> Aucun port détecté')
    else:
        objappli.b_Connecter['state'] = 'normal'
        ecrire_log(objappli,'>>> Port(s) détectés')

def connecter(objappli,objserie):
    """ Fonction se connectant au port selectionné """
    if(objappli.cbx_listeports != 0):
        varPorts = objappli.cbx_listeports.get()
        objserie.serialPortSelected = varPorts[3:5]
        ecrire_log(objappli,">>> Port USB sélectionné: "+str(objserie.serialPortSelected))
        errk = objserie.connection()
        if(errk == 1):
            objappli.var_statutnucleo.set("Connecté à COM"+objserie.serialPortSelected)
            ecrire_log(objappli,"Connexion Nucléo établie")
        elif(errk == -1):
            objappli.var_statutnucleo.set("Pas de connexion")
            ecrire_log(objappli,"Connexion Nucléo échouée")

def envoi_car(objappli,objserie):
    car=str(objappli.e_saisie.get())
    """ Fonction permettant d'envoyer le caractère écrit sur l'application vers la Nucleo"""
    ecrire_log(objappli,">>> caractère envoyé: "+car)
    ok=objserie.Nucleo_envoi_car(car)
    if ok==1:
        ecrire_log(objappli,">>> Message reçu")
    else:
        ecrire_log(objappli,">>> Message non transmis")

def envoi_spec(objappli,objserie,car):
    """" Fonction permettant d'envoyer un caractère spécifique passé en argument à la nucléo """
    ecrire_log(objappli,">>> caractère envoyé: "+car)
    ok=objserie.Nucleo_envoi_car(car)
    if ok==1:
        ecrire_log(objappli,">>> Message reçu")
    else:
        ecrire_log(objappli,">>> Message non transmis")

def executer(objappli,objserie):
    """ Fonction appelée par le bouton Executer """
    liste_instruction=objappli.cheminrobot
    ecrire_log(objappli,"a")
    for caractere in liste_instruction:
        envoi_spec(objappli,objserie,caractere)
        time.sleep(8)
        # fini=False
        # while not fini:
        #     time.sleep(0.5)
        #     statut=objserie.Nucleo_recu_car()
        #     if statut=='f':
        #         fini=True

## Gestion du calcul du meilleur chemin

def calcul_chemin(obj):
    effacer_chemin(obj)
    ecrire_log(obj,">>> Calcul du meilleur chemin")
    obj.chemin=PathFinding.find_shortest_path(obj.mapbase,obj.case_depart,obj.case_arrivee)
    obj.chemin=obj.chemin[1:]
    obj.chemin=obj.chemin[::-1]
    ecrire_log(obj,">>> Meilleur chemin détecté:")
    ecrire_log(obj,str(obj.chemin))
    for coord_case in obj.chemin[1:-1]:
        obj.carte.itemconfigure(obj.carte.liste_cases[coord_case[0]][coord_case[1]],fill='yellow')

def calcul_ch_absolu(obj):
    chemincarte=obj.chemin
    cheminabsolu=TraductionChemin.chemin_refcarte(chemincarte)
    obj.cheminabsolu=cheminabsolu
    obj.var_chemabsolu.set(str(cheminabsolu))

def calcul_ch_robot(obj):
    cheminabsolu=obj.cheminabsolu
    cheminrobot=TraductionChemin.commande_refrobot(cheminabsolu)
    cheminfin=TraductionChemin.commande_finale(cheminrobot)
    obj.cheminrobot=cheminfin
    obj.var_chemrobot.set(str(cheminfin))

## Manipulation d'images

def charger_image(chemin,taille):
    """ Fonction permettant de charger une image depuis un fichier de façon a être utilisée dans une interface tkinter
    Entrée: chemin=l'adresse du fichier image,taille= la taille (pixels) désirée pour l'image"""
    I=PIL.Image.open(chemin)
    I=I.resize(taille,PIL.Image.ANTIALIAS)
    return PIL.ImageTk.PhotoImage(I)