#-----------------------------------------------------------------------------------#
#                                                                                   #
#                           PROTIS 2021-2022                                        #
#                                                                                   #
#-----------------------------------------------------------------------------------#
#                           Etienne LOISELET                                        #
#-----------------------------------------------------------------------------------#

# Institut d'Optique Graduate School (https://www.institutoptique.fr/)
# LEnsE (http://lense.institutoptique.fr/)

# Il peut s'avérer utile de lire le guide d'utilisation avant la lecture de ce code

# Au 2A qui veut utiliser ce programme et ses librairies: en cas de question(s) ne pas
# hésiter à me contacter par mail à etienne.loiselet@institutoptique.fr ou sur messenger

## Importations:

import Actions                  # Librairie spécifique au projet (contient les actions des boutons)
import ConnectionNucleo         # Librairie spécifique au projet (gère la connexion avec la Nucléo)

import tkinter as tk            # Librairie générale de python (interfaçage)
import tkinter.ttk as ttk       # Librairie générale de python (options supplémentaires)
import tkinter.font as tkFont   # Librairie générale de python (polices de caractère)
import numpy as np              # Librairie générale de python (manipulation de matrices)
import threading                # Librairie générale de python (actions par interruption)
import importlib                # Librairie générale de python (permet de mettre à jour une librairie)

importlib.reload(Actions)           # mise à jour de "Actions" et "Connection Nucleo" pour appliquer
importlib.reload(ConnectionNucleo)  # les potentielles modifications sans avoir à tout redémarrer

## Classe définissant l'application:

# Si c'est la première fois que vous rencontrez une classe, profitez des tutoriels "Introduction à la programation orientée objet"
# et "Faire une interface avec Tkinter" disponibles dans le rapport technique pour mieux comprendre ce code

class Logiciel(tk.Tk):
    """ Classe génèrant un objet de type Tk (hérité de Tkinter), c'est le support qui contiendra l'entièreté
    de l'application. Il faut avoir défini un fond de carte au préalable. Ce fond de carte doit être de type np.array
    Pour lancer l'application, exécuter les commandes suivantes (présentes dans le fichier "Main.py"):
    XX = Logiciel(fond)
    XX.ouvrir()
    """
    def __init__(self,fond_de_carte,hauteur=700):
        """ hauteur est un paramètre prédéfini mais que l'on peut changer lors de la création de
        l'application par exemple:
        YY = Application_debug(fond,hauteur=300)"""
        # Initialisation:
        tk.Tk.__init__(self)                        # On initialise à partir de la classe tk.Tk
        self.mapbase=fond_de_carte                  # Fond de carte du hangar (0=traversable, 1=mur)
        self.n_l,self.n_c=np.shape(fond_de_carte)   # Nombre de lignes et de colonnes de la carte
        self.hcarte=450                             # Hauteur de la carte en pixel
        self.t=int(self.hcarte/self.n_l)            # Taille d'une case en pixel
        self.case_depart=(2,2)                      # Coordonnées de la case de départ (origine à (0,0))
        self.case_arrivee=(5,8)                     # Coordonnées de la case d'arrivée
        self.case_station=(0,6)                     # Coordonnées de la case de la station
        self.case_actu=(0,0)                        # Coordonnées de la dernière case sélectionnée
        self.chemin=[]                              # Liste des coordonnées des cases du chemin actuel
        self.cheminabsolu=[]                        # Liste des instructions en référentiel absolu
        self.cheminrobot=[]                         # Liste des instructions en référentiel relatif

        # Variables importantes
        # /!\/!\/!\ /!\/!\/!\ /!\/!\/!\
        # /!\/!\/!\ ATTENTION /!\/!\/!\
        # /!\/!\/!\ /!\/!\/!\ /!\/!\/!\
        # Veuillez renseigner dans chemin_g le chemin d'accès au fichier contenant les images du projet
        # (disponible dans le rapport technique
        chemin_g="D:/Documents/Etudes/2021-2022 -- SupOptique 2AP/Cours/ProTIS/Images/"

        # Connection Nucléo:
        self.objet_serie=ConnectionNucleo.SerialUSB_Nucleo()    # Création de l'objet de liaison série PC-Nucléo1
        self.ports=self.objet_serie.serial_ports()              # Liste des ports du PC susceptibles d'être connecté à une Nucléo

        # Paramétres visuels de la fenêtre:         (tailles,titre et icône)
        nb_or=(1+5**(1/2))/2
        self.height=hauteur
        self.width=int(self.height*nb_or)
        self.shape=str(self.width)+"x"+str(self.height)
        self.geometry(self.shape)
        self.title("Application ProTIS Version 1.0")
        self.iconbitmap(chemin_g+"Icone_Solec.ico")

        # Polices utilisées
        self.police_titre=tkFont.Font(family="Helvetica",size=36,weight='bold')
        self.police_souligne=tkFont.Font(family="Helvetica",underline=1)
        self.police_legende=tkFont.Font(family="Helvetica",slant='italic',size=9)

        # Images utilisées
        self.logo_Solec=Actions.charger_image(chemin_g+"Logo_Solec_sf.png",(350,80))
        self.logo_IOGS=Actions.charger_image(chemin_g+"Logo_IOGS_sf.png",(350,80))
        taille=(20,20)
        self.loupe=Actions.charger_image(chemin_g+"icone_recherche.png",taille)
        self.avion=Actions.charger_image(chemin_g+"icone_envoyer.png",taille)
        self.I_tgauche=Actions.charger_image(chemin_g+"icone_tgauche.png",taille)
        self.I_tdroite=Actions.charger_image(chemin_g+"icone_tdroite.png",taille)
        self.I_gauche=Actions.charger_image(chemin_g+"icone_gauche.png",taille)
        self.I_droite=Actions.charger_image(chemin_g+"icone_droite.png",taille)
        self.I_haut=Actions.charger_image(chemin_g+"icone_haut.png",taille)
        self.I_bas=Actions.charger_image(chemin_g+"icone_bas.png",taille)
        self.I_pointcard=Actions.charger_image(chemin_g+"icone_pointscard.png",(60,60))
        self.I_robot=Actions.charger_image(chemin_g+"icone_robot.png",(100,100))
        self.I_robotlegende=Actions.charger_image(chemin_g+"icone_robot.png",(17,17))
        self.I_nucleo=Actions.charger_image(chemin_g+"icone_nucleo.png",(100,100))

        # Définition des evenements de threading (action par interruption)
        self.event=threading.Event()       # Evenement "Une case a été selectionnée"

    def creation(self):
        """ Fonction interne à la classe qui crée tous les objets destinés à être affichés sur la fenêtre
        Se réferer au Guide d'Utilisation pour une liste détaillée des objets"""

        # Frame 1: Log
        self.f1=tk.Frame(self,bd=2,relief='groove')                             # Cadre d'insertion de la partie f1
            # Frame 1.1: Texte et Scrollbar
        self.f11=tk.Frame(self.f1,relief='groove')                              # Cadre d'insertion du log
        self.log=tk.Text(self.f11,width=30,height=1)                            # Zone de texte du log
        self.log.insert('1.0',">>> Début du log de commandes")                  # On écrit un message de bienvenue
        self.log.configure(state='disabled')                                    # On empêche l'utilisateur d'écrire sur le log
        self.y_scroll=tk.Scrollbar(self.f11,orient="vertical",command=self.log.yview) # Scrollbar du log
        self.log['yscrollcommand']=self.y_scroll.set                            # On lie la scrollbar à la zone de texte
            #Frame 1.2: Boutons Effacer et Quitter
        self.f12=tk.Frame(self.f1,relief='groove')                              # Cadre d'insertion des boutons
        self.b_effacer=tk.Button(self.f12,text="Effacer le log",command= lambda: Actions.effacer_log(self)) # Bouton effacer
        self.b_quitter=tk.Button(self.f12,text="Fermer",command=self.fermer)    # Bouton quitter

        # Frame 2: Titre
        self.f2=tk.Frame(self,bd=2,relief='groove')                             # Cadre d'insertion de la partie f2
        self.l_logo_Solec=tk.Label(self.f2,image=self.logo_Solec)               # Logo de Solec
        self.l_titre=tk.Label(self.f2,font=self.police_titre,text="Super Programme") # Titre central
        self.l_logo_IOGS=tk.Label(self.f2,image=self.logo_IOGS)                 # Logo de l'IOGS

        # Frame 3: Zone de dessin de la carte
        self.f3=tk.Frame(self,bd=2,relief='groove')                             # Cadre d'insertion de la partie f3
        self.carte=Actions.Canvas_Perso(self,self.f3,self.hcarte,self.t,self.n_c,self.n_l) # Zone de dessin à partir d'une classe personnalisée

        # Frame 4: Lien Robot
        self.f4=tk.Frame(self,bd=2,relief='groove')                             # Cadre d'insertion de la partie f4
            # Frame 4.1: Connexion Robot
        self.f41=tk.Frame(self.f4,bd=2,relief='groove')                         # Cadre d'insertion de la partie f41
                # Frame 4.1.1: Retour Robot
        self.f411=tk.Frame(self.f41,bd=2,relief='groove')                       # Cadre d'insertion de la partie f411
        self.l_imagerobot=tk.Label(self.f411,image=self.I_robot)                # Image du robot
        # A completer: affichage du retour robot
        self.l_vitesse_titre=tk.Label(self.f411,text='Vitesse: ')               # Les 4 affichages du retour robot
        self.l_position_titre=tk.Label(self.f411,text='Position: ')
        self.l_prochaction_titre=tk.Label(self.f411,text='Prochaine action: ')
        self.l_actionactu_titre=tk.Label(self.f411,text='Action en cours: ')
        self.var_vitesse=tk.StringVar()                                         # Variables d'affichage contenant les retour du robot
        self.var_vitesse.set('X')
        self.var_position=tk.StringVar()
        self.var_position.set('X')
        self.var_prochaction=tk.StringVar()
        self.var_prochaction.set('X')
        self.var_actionactu=tk.StringVar()
        self.var_actionactu.set('X')
        self.l_vitesse=tk.Label(self.f411,textvariable=self.var_vitesse)
        self.l_position=tk.Label(self.f411,textvariable=self.var_position)
        self.l_prochaction=tk.Label(self.f411,textvariable=self.var_prochaction)
        self.l_actionactu=tk.Label(self.f411,textvariable=self.var_actionactu)
                # Frame 4.1.2: Statut Robot
        self.f412=tk.Frame(self.f41,bd=2,relief='groove')                       # Cadre d'insertion de la partie f411
        self.l_statutrobot_titre=tk.Label(self.f412,text='Statut: ')            # Affichage du statut de la connexion avec le robot
        self.var_statutrobot=tk.StringVar()
        self.var_statutrobot.set('Aucune connection')
        self.l_statutrobot=tk.Label(self.f412,textvariable=self.var_statutrobot)
        self.l_completion=tk.Label(self.f412,text='Complétion: ')
        # TO DO : add progress bar
            # Frame 4.2: Connexion Nucléo
        self.f42=tk.Frame(self.f4,bd=2,relief='groove')                         # Cadre d'insertion de la partie f42
                # Frame 4.2.1: Statut Nucléo
        self.f421=tk.Frame(self.f42,bd=2,relief='groove')                       # Cadre d'insertion de la partie f421
        self.l_imagenucleo=tk.Label(self.f421,image=self.I_nucleo)              # Image de la carte nucléo
        self.l_ports=tk.Label(self.f421,text="Ports: ")                         # Affichage des ports
        self.l_statutnucleo_titre=tk.Label(self.f421,text='Statut: ')           # Affichage du statut de la connexion avec la Nucléo 1
        self.var_statutnucleo=tk.StringVar()
        self.var_statutnucleo.set('Aucune connection')
        self.l_statutnucleo=tk.Label(self.f421,textvariable=self.var_statutnucleo)
        self.b_TrouverPorts=tk.Button(self.f421,image=self.loupe,command=lambda:Actions.trouver_ports(self,self.objet_serie)) # Bouton de recherche des ports utilisés
        self.b_Connecter=tk.Button(self.f421,text='Connecter',command=lambda:Actions.connecter(self,self.objet_serie)) # Bouton de connexion au port selectionné
        self.cbx_listeports=ttk.Combobox(self.f421)
                # Frame 4.2.2: Envoi Nucléo
        self.f422=tk.Frame(self.f42,bd=2,relief='groove')                       # Cadre d'insertion de la partie f422
        self.e_saisie=tk.Entry(self.f422,width='5')                             # Zone d'insertion du caractère à envoyer à la nucléo
        self.l_saisie=tk.Label(self.f422,text='Caractère à envoyer: ')
        self.b_envoicar=tk.Button(self.f422,image=self.avion,command=lambda:Actions.envoi_car(self,self.objet_serie)) # Bouton d'envoi du caractère
            # Frame 4.3: Contrôle Robot
        self.f43=tk.Frame(self.f4,bd=2,relief='groove')                         # Cadre d'insertion de la partie f43
                # Frame 4.3.1: Mode du Robot
        self.f431=tk.Frame(self.f43,bd=2,relief='groove')                       # Cadre d'insertion de la partie f431
        # Partie à compléter
        self.l_Mode_titre=tk.Label(self.f431,text='Mode: ')
        self.var_mode=tk.StringVar()
        self.var_mode.set('Automatique')
        self.l_mode=tk.Label(self.f431,textvariable=self.var_mode)
        self.var_mode_bouton=tk.StringVar()
        self.var_mode_bouton.set('Passer en mode Manuel')
        self.b_mode=tk.Button(self.f431,textvariable=self.var_mode_bouton,command=None)
                # Frame 4.3.2: Boutons de contrôle du Robot
        # Partie à compléter
        self.f432=tk.Frame(self.f43,bd=2,relief='groove')
        self.b_tourgauche=tk.Button(self.f432,image=self.I_tgauche,command=None)
        self.b_haut=tk.Button(self.f432,image=self.I_haut,command=None)
        self.b_tourdroite=tk.Button(self.f432,image=self.I_tdroite,command=None)
        self.b_gauche=tk.Button(self.f432,image=self.I_gauche,command=None)
        self.b_bas=tk.Button(self.f432,image=self.I_bas,command=None)
        self.b_droite=tk.Button(self.f432,image=self.I_droite,command=None)

        # Frame 5: Actions
        self.f5=tk.Frame(self,bd=2,relief='groove')                             # Cadre d'insertion de la partie f5
            # Frame 5.1: Légende de la carte
        self.f51=tk.Frame(self.f5,bd=2,relief='groove')                         # Cadre d'insertion de la partie f51
        # Création de tous les objets de la légende
        self.l_legende=tk.Label(self.f51,text='Légende:',font=self.police_souligne)
        self.canvas_traversable=Actions.Case_Legende(self.f51,'white')          # Se référer à la librairie Actions pour voir le détail
        self.l_case_traversable=tk.Label(self.f51,text='Traversable',font=self.police_legende)
        self.canvas_mur=Actions.Case_Legende(self.f51,'black')
        self.l_case_mur=tk.Label(self.f51,text='Mur',font=self.police_legende)
        self.canvas_depart=Actions.Case_Legende(self.f51,'green')
        self.l_case_depart=tk.Label(self.f51,text='Départ',font=self.police_legende)
        self.canvas_arrivee=Actions.Case_Legende(self.f51,'red')
        self.l_case_arrivee=tk.Label(self.f51,text='Arrivée',font=self.police_legende)
        self.canvas_station=Actions.Case_Legende(self.f51,'cyan')
        self.l_case_station=tk.Label(self.f51,text='Station',font=self.police_legende)
        self.canvas_chemin=Actions.Case_Legende(self.f51,'yellow')
        self.l_case_chemin=tk.Label(self.f51,text='Chemin',font=self.police_legende)
        self.l_robotlegende=tk.Label(self.f51,image=self.I_robotlegende)
        self.l_robottexte=tk.Label(self.f51,text='Robot',font=self.police_legende)
        self.l_cardinaux=tk.Label(self.f51,image=self.I_pointcard)
            # Frame 5.2: Boutons pour la carte
        self.f52=tk.Frame(self.f5,bd=2,relief='groove')                         # Cadre d'insertion de la partie f52
        # Création des divers boutons
        self.b_depart=tk.Button(self.f52,text="Définir départ",command=lambda: Actions.def_depart(self))
        self.b_arrivee=tk.Button(self.f52,text="Définir arrivée",command=lambda: Actions.def_arrivee(self))
        self.b_station=tk.Button(self.f52,text="Définir station",command=lambda: Actions.def_station(self))
        self.b_reinit=tk.Button(self.f52,text="Réinitialiser case",command=lambda: Actions.reinit_case(self))
        self.b_chemin=tk.Button(self.f52,text="Calcul chemin",command=lambda: Actions.calcul_chemin(self))
        self.b_effchem=tk.Button(self.f52,text="Effacer chemin",command=lambda: Actions.effacer_chemin(self))
        self.l_passestation=tk.Label(self.f52,text='Passer par la station')
        # TO DO tickbox
        self.b_ouvrircarte=tk.Button(self.f52,text='Ouvrir une carte',command=None)
        self.b_creercarte=tk.Button(self.f52,text='Créer une carte',command=None)
        self.b_lancer_chemin=tk.Button(self.f52,text='Executer',command=lambda:Actions.executer(self,self.objet_serie))
            # Frame 5.3: Boutons pour le chemin
        self.f53=tk.Frame(self.f5,bd=2,relief='groove')                         # Cadre d'insertion de la partie f53
        self.l_chemabsolu_titre=tk.Label(self.f53,text='Chemin absolu: ')
        self.l_chemrobot_titre=tk.Label(self.f53,text='Chemin robot: ')
        self.var_chemabsolu=tk.StringVar()
        self.var_chemabsolu.set('Aucun chemin en cours')
        self.var_chemrobot=tk.StringVar()
        self.var_chemrobot.set('Aucun chemin en cours')
        self.l_chemabsolu=tk.Label(self.f53,textvariable=self.var_chemabsolu)
        self.l_chemrobot=tk.Label(self.f53,textvariable=self.var_chemrobot)
        self.b_calculabsolu=tk.Button(self.f53,text='Calculer',command=lambda:Actions.calcul_ch_absolu(self))
        self.b_calculrobot=tk.Button(self.f53,text='Calculer',command=lambda:Actions.calcul_ch_robot(self))

    def placement(self):
        """ Fonction interne à la classe qui place au bon endroit tous les objets créés dans la fenêtre
        """
        # Définition de la grille de placement dans la fenêtre:
        self.rowconfigure(0,weight=0)
        self.rowconfigure(1,weight=0)
        self.rowconfigure(2,weight=1)
        self.columnconfigure(0,weight=0)
        self.columnconfigure(1,weight=1)
        self.columnconfigure(2,weight=1)

        # Définition de la grille de placement dans Frame 1:
        self.f1.columnconfigure(0,weight=1)
        self.f1.rowconfigure(0,weight=1)
        self.f1.rowconfigure(1,weight=0)
        # Placement Frame 1:
        self.f1.grid(row=1,column=2,rowspan=2,sticky='nsew')
            # Définition de la grille de placement dans Frame 1.1:
        self.f11.rowconfigure(0,weight=1)
        self.f11.columnconfigure(0,weight=1)
        self.f11.columnconfigure(1,weight=0)
            # Placement Frame 1.1:
        self.f11.grid(row=0,column=0,sticky='nsew')
        self.log.grid(row=0,column=0,sticky='nsew')
        self.y_scroll.grid(row=0,column=1,sticky='ns')
            # Définition de la grille de placement dans Frame 1.2:
        self.f12.columnconfigure(0,weight=1)
        self.f12.columnconfigure(1,weight=1)
            # Placement Frame 1.2:
        self.f12.grid(row=1,column=0,sticky='ew')
        self.b_effacer.grid(row=0,column=0)
        self.b_quitter.grid(row=0,column=1)

        # Définition de la grille de placement dans Frame 2:
        self.f2.rowconfigure(0,weight=1)
        self.f2.columnconfigure(0,weight=1)
        self.f2.columnconfigure(1,weight=0)
        self.f2.columnconfigure(2,weight=1)
        # Placement Frame 2:
        self.f2.grid(row=0,column=0,columnspan=3,sticky='nsew')
        self.l_logo_Solec.grid(row=0,column=0,sticky='nsew')
        self.l_titre.grid(row=0,column=1)
        self.l_logo_IOGS.grid(row=0,column=2,sticky='nsew')

        # Définition de la grille de placement dans Frame 3:
        self.f3.rowconfigure(0,weight=1)
        self.f3.columnconfigure(0,weight=0)
        self.f3.columnconfigure(1,weight=1)
        # Placement Frame 3:
        self.f3.grid(row=1,column=1,sticky='nsew')
        self.carte.grid(row=0,column=1)

        # Définition de la grille de placement dans Frame 4:
        self.f4.rowconfigure(0,weight=1)
        self.f4.rowconfigure(1,weight=1)
        self.f4.rowconfigure(2,weight=1)
        self.f4.columnconfigure(0,weight=1)
        # Placement Frame 4:
        self.f4.grid(row=1,column=0,rowspan=2,sticky='nsew')
            # Définition de la grille de placement dans Frame 4.1:
        self.f41.rowconfigure(0,weight=1)
        self.f41.rowconfigure(1,weight=1)
        self.f41.columnconfigure(0,weight=1)
            # Placement Frame 4.1:
        self.f41.grid(row=0,column=0,sticky='nsew')
                # Définition de la grille de placement dans Frame 4.1.1:
        self.f411.rowconfigure(0,weight=1)
        self.f411.rowconfigure(1,weight=1)
        self.f411.rowconfigure(2,weight=1)
        self.f411.rowconfigure(3,weight=1)
        self.f411.columnconfigure(0,weight=1)
        self.f411.columnconfigure(1,weight=1)
        self.f411.columnconfigure(2,weight=1)
                # Placement Frame 4.1.1:
        self.f411.grid(row=0,column=0,sticky='nsew')
        self.l_imagerobot.grid(row=0,column=0,rowspan=4)
        self.l_vitesse_titre.grid(row=0,column=1)
        self.l_position_titre.grid(row=1,column=1)
        self.l_prochaction_titre.grid(row=2,column=1)
        self.l_actionactu_titre.grid(row=3,column=1)
        self.l_vitesse.grid(row=0,column=2)
        self.l_position.grid(row=1,column=2)
        self.l_prochaction.grid(row=2,column=2)
        self.l_actionactu.grid(row=3,column=2)
                # Définition de la grille de placement dans Frame 4.1.2:
        self.f412.rowconfigure(0,weight=1)
        self.f412.rowconfigure(1,weight=1)
        self.f412.rowconfigure(2,weight=1)
        self.f412.columnconfigure(0,weight=1)
        self.f412.columnconfigure(1,weight=1)
                # Placement Frame 4.1.2:
        self.f412.grid(row=1,column=0,sticky='nsew')
        self.l_statutrobot_titre.grid(row=0,column=0)
        self.l_statutrobot.grid(row=0,column=1)
        self.l_completion.grid(row=1,column=0,columnspan=2)
            # Définition de la grille de placement dans Frame 4.2:
        self.f42.rowconfigure(0,weight=1)
        self.f42.rowconfigure(1,weight=1)
        self.f42.columnconfigure(0,weight=1)
            # Placement Frame 4.2:
        self.f42.grid(row=1,column=0,sticky='nsew')
                # Définition de la grille de placement dans Frame 4.2.1:
        self.f421.rowconfigure(0,weight=1)
        self.f421.rowconfigure(1,weight=1)
        self.f421.rowconfigure(2,weight=1)
        self.f421.columnconfigure(0,weight=1)
        self.f421.columnconfigure(1,weight=1)
        self.f421.columnconfigure(2,weight=1)
        self.f421.columnconfigure(3,weight=1)
                # Placement Frame 4.2.1:
        self.f421.grid(row=0,column=0,sticky='nsew')
        self.l_imagenucleo.grid(row=0,column=0,rowspan=3)
        self.l_ports.grid(row=0,column=1)
        self.l_statutnucleo_titre.grid(row=1,column=1)
        self.l_statutnucleo.grid(row=1,column=2)
        self.b_TrouverPorts.grid(row=0,column=3)
        self.b_Connecter.grid(row=2,column=1,columnspan=3)
        self.cbx_listeports.grid(row=0,column=2)
                # Définition de la grille de placement dans Frame 4.2.2:
        self.f422.rowconfigure(0,weight=1)
        self.f422.columnconfigure(0,weight=1)
        self.f422.columnconfigure(1,weight=1)
        self.f422.columnconfigure(2,weight=0)
                # Placement Frame 4.2.2:
        self.f422.grid(row=1,column=0,sticky='nsew')
        self.l_saisie.grid(row=0,column=0)
        self.e_saisie.grid(row=0,column=1)
        self.b_envoicar.grid(row=0,column=2)
            # Définition de la grille de placement dans Frame 4.3:
        self.f43.rowconfigure(0,weight=1)
        self.f43.rowconfigure(1,weight=1)
        self.f43.columnconfigure(0,weight=1)
            # Placement Frame 4.3:
        self.f43.grid(row=2,column=0,sticky='nsew')
                # Définition de la grille de placement dans Frame 4.3.1:
        self.f431.rowconfigure(0,weight=1)
        self.f431.rowconfigure(1,weight=1)
        self.f431.columnconfigure(0,weight=1)
        self.f431.columnconfigure(1,weight=1)
                # Placement Frame 4.3.1:
        self.f431.grid(row=0,column=0,sticky='nsew')
        self.l_Mode_titre.grid(row=0,column=0)
        self.l_mode.grid(row=0,column=1)
        self.b_mode.grid(row=1,column=0,columnspan=2)
                # Définition de la grille de placement dans Frame 4.3.2:
        self.f432.rowconfigure(0,weight=1)
        self.f432.rowconfigure(1,weight=1)
        self.f432.columnconfigure(0,weight=1)
        self.f432.columnconfigure(1,weight=1)
        self.f432.columnconfigure(2,weight=1)
                # Placement Frame 4.3.2:
        self.f432.grid(row=1,column=0,sticky='nsew')
        self.b_tourgauche.grid(row=0,column=0,sticky='nsew')
        self.b_haut.grid(row=0,column=1,sticky='nsew')
        self.b_tourdroite.grid(row=0,column=2,sticky='nsew')
        self.b_gauche.grid(row=1,column=0,sticky='nsew')
        self.b_bas.grid(row=1,column=1,sticky='nsew')
        self.b_droite.grid(row=1,column=2,sticky='nsew')

        # Définition de la grille de placement dans Frame 5:
        self.f5.rowconfigure(0,weight=0)
        self.f5.rowconfigure(1,weight=1)
        self.f5.columnconfigure(0,weight=1)
        self.f5.columnconfigure(1,weight=0)
        # Placement Frame 5:
        self.f5.grid(row=2,column=1,sticky='nsew')
            # Définition de la grille de placement dans Frame 5.1:
        self.f51.rowconfigure(0,weight=1)
        self.f51.rowconfigure(1,weight=1)
        self.f51.rowconfigure(2,weight=1)
        self.f51.columnconfigure(0,weight=1)
        self.f51.columnconfigure(1,weight=1)
        self.f51.columnconfigure(2,weight=1)
        self.f51.columnconfigure(3,weight=1)
        self.f51.columnconfigure(4,weight=1)
        self.f51.columnconfigure(5,weight=1)
        self.f51.columnconfigure(6,weight=1)
            # Placement Frame 5.1:
        self.f51.grid(row=0,column=1)
        self.l_legende.grid(row=0,column=0,columnspan=2,sticky='ew')
        self.canvas_traversable.grid(row=1,column=0)
        self.l_case_traversable.grid(row=1,column=1,sticky='ew')
        self.canvas_mur.grid(row=2,column=0)
        self.l_case_mur.grid(row=2,column=1,sticky='ew')
        self.canvas_depart.grid(row=1,column=2)
        self.l_case_depart.grid(row=1,column=3,sticky='ew')
        self.canvas_arrivee.grid(row=2,column=2)
        self.l_case_arrivee.grid(row=2,column=3,sticky='ew')
        self.canvas_station.grid(row=1,column=4)
        self.l_case_station.grid(row=1,column=5,sticky='ew')
        self.canvas_chemin.grid(row=2,column=4)
        self.l_case_chemin.grid(row=2,column=5,sticky='ew')
        self.l_robotlegende.grid(row=1,column=6)
        self.l_robottexte.grid(row=1,column=7,sticky='ew')
        self.l_cardinaux.grid(row=0,column=8,rowspan=3)
            # Définition de la grille de placement dans Frame 5.2:
        self.f52.rowconfigure(0,weight=1)
        self.f52.rowconfigure(1,weight=1)
        self.f52.rowconfigure(2,weight=1)
        self.f52.rowconfigure(3,weight=1)
        self.f52.columnconfigure(0,weight=1)
        self.f52.columnconfigure(1,weight=1)
        self.f52.columnconfigure(2,weight=1)
            # Placement Frame 5.2:
        self.f52.grid(row=0,column=0,rowspan=2,sticky='nsew')
        self.b_depart.grid(row=0,column=0)
        self.b_arrivee.grid(row=1,column=0)
        self.b_station.grid(row=0,column=1)
        self.b_reinit.grid(row=1,column=1)
        self.b_chemin.grid(row=0,column=2)
        self.b_effchem.grid(row=1,column=2)
        self.l_passestation.grid(row=2,column=0,columnspan=2)
        self.b_ouvrircarte.grid(row=3,column=0)
        self.b_creercarte.grid(row=3,column=1)
        self.b_lancer_chemin.grid(row=3,column=2)
            # Définition de la grille de placement dans Frame 5.3:
        self.f53.rowconfigure(0,weight=1)
        self.f53.rowconfigure(1,weight=1)
        self.f53.columnconfigure(0,weight=1)
        self.f53.columnconfigure(1,weight=1)
        self.f53.columnconfigure(2,weight=1)
            # Placement Frame 5.3:
        self.f53.grid(row=1,column=1,sticky='nsew')
        self.l_chemabsolu_titre.grid(row=0,column=0)
        self.l_chemrobot_titre.grid(row=1,column=0)
        self.l_chemabsolu.grid(row=0,column=1)
        self.l_chemrobot.grid(row=1,column=1)
        self.b_calculabsolu.grid(row=0,column=2)
        self.b_calculrobot.grid(row=1,column=2)

    def ouvrir(self):
        """ Fonction à appeler pour pouvoir lancer l'application d'un seul coup (plus facile pour l'utilisateur
        qui ne connait pas le programme en détail)
        """
        self.creation()
        self.placement()
        self.mainloop()

    def fermer(self):
        self.destroy()