#-----------------------------------------------------------------------------------#
#                                                                                   #
#                           PROTIS 2021-2022                                        #
#                                                                                   #
#-----------------------------------------------------------------------------------#
#                           Etienne LOISELET                                        #
#-----------------------------------------------------------------------------------#

# Cette librairie est très inspirée de la librairie SerialUSB créée par Julien Villemejane
# sur le site du LEnsE
# Institut d'Optique Graduate School (https://www.institutoptique.fr/)
# LEnsE (http://lense.institutoptique.fr/)

# Au 2A qui veut utiliser ce programme et ses librairies: en cas de question(s) ne pas
# hésiter à me contacter par mail à etienne.loiselet@institutoptique.fr ou sur messenger

# ATTENTION ATTENTION Pour executer ce code il faut avoir installé la librairie pyserial
# au préalable (non native à python) ET SURTOUT PAS serial !!!!
# pour installer une librairie veuillez taper: pip install "nom de la librairie", sur l'invite de commande
# ici veuillez taper: pip install pyserial

# Si serial est déjà installé veuillez taper: pip uninstall serial, éteindre et rallumer le compilateur puis installer pyserial

## Importations;
import serial.tools                 # Suite des importations
import serial.tools.list_ports      # Suite des importations
import time                         # gestion du temps
import serial                       # Une librairie a importer

## Classe:
class SerialUSB_Nucleo:
    ''' Classe permettant de créer un objet virtuel représentant la liaison série entre le PC et la nucleo'''
    def __init__(self):                                     # Initialisation de la classe et de ses paramètres
        self.nb_port = 0                                    # Nombre de ports détectés (initialisé à 0)
        self.list_port = []                                 # Liste des ports détectés (initialement vide)
        self.serialPortSelected = 0                         # Numéro du port sélectionné (initialement 0)

    def serial_ports(self):
        """Fonction interne à la classe SerialUSB_Nucleo qui recalcule (destructif) le nombre et la liste de ports
           disponibles. Ne détecte que les ports comprenant 'STM' ou 'USB' dans leur nom
           ENTREE: Aucune
           SORTIE: la liste des ports disponibles (0 si vide). Attention les paramètres .nb_port et .list_port de l'objet
           ont été modifiés
        """
        self.nb_port = 0                                        # Réinitialisation du paramètre
        ports = list(serial.tools.list_ports.comports())        # Liste de tous les ports détectés
        self.list_port = []                                     # Réinitialisation du paramètre
        for port in ports:                                      # Boucle de sélection des ports intéressants
            if 'STM' in str(port) or 'USB' in str(port):
                self.nb_port += 1
                self.list_port.append(str(port))
        if(self.nb_port != 0):                                  # Procédure de renvoi en fonction des ports détectés
            return self.list_port
        else:
            return 0

    def connection(self):
        '''Fonction interne à la classe SerialUSB_Nucleo qui vérifie que la connection Python-Nucléo a bien été effectuée.
           L'ordinateur envoie le caractère "z" à la Nucléo puis attend 10 ms que la carte Nucléo lui renvoie le caractère "k".
           Si un "k" a été détecté la fonction renvoie 1 si 10 ms se sont écoulées avant, la fonction renvoie -1
           ENTREE: Aucune
           SORTIE: entier de valeur 1 si la connection est bien établie, -1 sinon
           ATTENTION il faut avoir mis du code dans la nucléo pour qu'elle renvoie un "k" si elle reçoit un "z" !!!!! Sinon
           l'ordinateur ne sera jamais au courant que la nucléo a bien reçu l'instruction.
        '''
        serialSelected = serial.Serial('COM'+ self.serialPortSelected, 115200, timeout=1)
        if(serialSelected.isOpen() == False):
            serialSelected.open()                       # Ouverture du port selectionné
        serialSelected.write(bytes('z','utf-8'))        # Envoi d'un "z"
        dataRecOk = 0                                   # Pour l'instant aucun retour de la Nucléo
        timeoutRec = 0                                  # Initialisation du chronomètre
        while (dataRecOk == 0) or (timeoutRec < 10):    # Tant que pas de retour de la Nucléo et que temps pas écoulé
            timeoutRec += 1                             # On avance le chrono
            if(timeoutRec > 10):                        # Le temps max est-il écoulé
                return -1                               # On entre dans la boucle s'il est trop tard donc on renvoie -1
            if serialSelected.inWaiting() > 0:          # On vérifie qu'il y a bien qqchose qui est arrivé au PC
                dataReceived = serialSelected.read(1)   # Si oui on lit ce qqchose
                if(dataReceived == bytes('y','utf-8')): # Si c'est un "y" on renvoie 1 et on sort de la boucle
                    dataRecOk = 1
                    return 1
            time.sleep(0.001)                           # On attend 1 ms avant de replonger dans la boucle (chrono)

    def Nucleo_envoi_car(self,car):
        '''Fonction interne à la classe SerialUSB_Nucleo qui envoie le caractère car à la carte Nucléo connectée. Il n'y a pas de
           retour pour vérifier que l'instruction a bien été reçue par la Nucléo
           ENTREE: un caractère car, le caractère à transmettre à la Nucléo
           SORTIE: aucune,le caractère à été envoyé à la Nucléo
        '''
        serialSelected = serial.Serial('COM'+ str(self.serialPortSelected), 115200, timeout=1)
        if(serialSelected.isOpen() == False):
            serialSelected.open()                       # Ouverture du port selectionné
        serialSelected.write(bytes(car,'utf-8'))    # Envoi de car
        dataRecOk = 0                                   # Pour l'instant aucun retour de la Nucléo
        timeoutRec = 0                                  # Initialisation du chronomètre
        while (dataRecOk == 0) or (timeoutRec < 10):    # Tant que pas de retour de la Nucléo et que temps pas écoulé
            timeoutRec += 1                             # On avance le chrono
            if(timeoutRec > 10):                        # Le temps max est-il écoulé
                return -1                               # On entre dans la boucle s'il est trop tard donc on renvoie -1
            if serialSelected.inWaiting() > 0:          # On vérifie qu'il y a bien qqchose qui est arrivé au PC
                dataReceived = serialSelected.read(1)   # Si oui on lit ce qqchose
                if(dataReceived == bytes('y','utf-8')): # Si c'est un "y" on renvoie 1 et on sort de la boucle
                    dataRecOk = 1
                    return 1
            time.sleep(0.001)                           # On attend 1 ms avant de replonger dans la boucle 55(chrono)

    def Nucleo_recu_car(self):
        '''Fonction interne à la classe SerialUSB_Nucleo qui envoie le caractère car à la carte Nucléo connectée. Il n'y a pas de
           retour pour vérifier que l'instruction a bien été reçue par la Nucléo
           ENTREE: un caractère car, le caractère à transmettre à la Nucléo
           SORTIE: aucune,le caractère à été envoyé à la Nucléo
        '''
        serialSelected = serial.Serial('COM'+ str(self.serialPortSelected), 115200, timeout=1)
        if(serialSelected.isOpen() == False):
            serialSelected.open()                       # Ouverture du port selectionné
        serialSelected.write(bytes('? ','utf-8'))    # Envoi de ?

        timeoutRec = 0                                  # Initialisation du chronomètre
        while (timeoutRec < 10):    # Tant que pas de retour de la Nucléo et que temps pas écoulé
            timeoutRec += 1                             # On avance le chrono
            if(timeoutRec > 10):                          # Le temps max est-il écoulé
                return -1                               # On entre dans la boucle s'il est trop tard donc on renvoie -1
            if serialSelected.inWaiting() > 0:          # On vérifie qu'il y a bien qqchose qui est arrivé au PC
                dataReceived = serialSelected.read(1)   # Si oui on lit ce qqchose
                return dataReceived
            time.sleep(0.001)                           # On attend 1 ms avant de replonger dans la boucle (chrono)