#-----------------------------------------------------------------------------------#
#                                                                                   #
#                           PROTIS 2021-2022                                        #
#                                                                                   #
#-----------------------------------------------------------------------------------#
#                           Grégoire GUYON                                          #
#-----------------------------------------------------------------------------------#

# Institut d'Optique Graduate School (https://www.institutoptique.fr/)
# LEnsE (http://lense.institutoptique.fr/)

import math as mp

def chemin_refcarte(parcours):   #parcours est une liste de tuples
  """Cette fonction permet de transformer la liste des cases à parcourir (parcours), donnée
  par l'algorithme du meilleur chemin, en instructions (haut bas gauche droite) par rapport à la carte (référentiel absolu).
  L'idée est d'utiliser les coordonnées des cases pour savoir à chaque fois la position relative de la case suivante
  par rapport à là où se trouve le robot.

  """
  taille=len(parcours)      #taille est un tuple de dimension 2
  depart=parcours[0]
  commandecarte=[]        #liste qui va contenir la suite des instructions dans le référentiel absolu sous forme ["H","B",...]
  for i in range(0,taille):           #Comme le robot ne peut pas se déplacer en diagonale, et qu'il se déplace d'une case par une case,
    if depart[0]>parcours[i][0]:      #Il suffit de faire un test sur les coordonées de la case suivant pour vérifier si elle est "au dessus", "à gauche" etc...
      commandecarte.append("H")
      depart=parcours[i]


    elif depart[0]<parcours[i][0] :    #parcours[i][0] renvoie la première coordonnées du tuple numéro i dans parcours
      commandecarte.append("B")
      depart=parcours[i]


    elif depart[1]<parcours[i][1] :
      commandecarte.append("D")
      depart=parcours[i]


    elif depart[1]>parcours[i][1] :
      commandecarte.append("G")
      depart=parcours[i]

  # print(commandecarte)
  return commandecarte


def commande_refrobot(commandecarte):
  """Cette fonction permet de transformer les instructions absolues
  en instruction relatives pour le robot qui ne sait que tourner et avancer.
  On utilise une variable "face" qui donne à chaque action la direction des roues du robot dans le repère absolu de la carte
  On compare l'instruction à "face" pour savoir s'il faut tourner ou avancer
  Pour se repérer il faut que le robot parte toujours de la même direction
    Nous avons choisi qu'il retourne toujours vers la position face vers le haut
  Une instruction à la fin de la fonction permet de ramener toujours le robot vers cette position. Il faut donc placer le robot par terre dans la bonne direction au début de l'utilisation
  """
  commande_robot=[]           #liste qui va contenir les commandes relatives sous forme "a,a,g,a,d"
  taille=len(commandecarte)
  face="H"
  for i in range(0,taille):
    if face==commandecarte[i]:
      commande_robot.append("a")      #ce caractère correspond à "avancer d'une case" CF annexe sur la signification des caractères


    else:
      if face=="H":
        if commandecarte[i]=="G":       #Comme le robot ne peut qu'avancer tout droit et tourner, s'il fait face vers le haut et doit aller à gauche d'une case (commande absolue "G")
          commande_robot.append("g")    #il doit tourner d'un quart de tour à gauche ('g' et avancer d'une case 'g')
          commande_robot.append("a")
          face="G"
        elif commandecarte[i]=="D" :
          commande_robot.append("d")
          commande_robot.append("a")
          face="D"
        else :
          commande_robot.append("&")    #Cas où il doit faire demi-tour
          commande_robot.append("a")
          face="B"


      elif face=="B":                 #On realise ce test simple pour toutes les configurations possibles
        if commandecarte[i]=="G":
          commande_robot.append("d")
          commande_robot.append(a)
          face="G"
        elif commandecarte[i]=="D":
          commande_robot.append("g")
          commande_robot.append("a")
          face="D"
        else :
          commande_robot.append("&")
          commande_robot.append("a")
          face="H"

      elif face=="G":
        if commandecarte[i]=="H":
          commande_robot.append("d")
          commande_robot.append("a")
          face="H"
        elif commandecarte[i]=="B":
          commande_robot.append("g")
          commande_robot.append("a")
          face="B"
        else :
          commande_robot.append("&")
          commande_robot.append("a")
          face="D"

      elif face=="D":
        if commandecarte[i]=="H":
          commande_robot.append("g")
          commande_robot.append("a")
          face="H"
        elif commandecarte[i]=="B":
          commande_robot.append("d")
          commande_robot.append("a")
          face="B"
        else :
          commande_robot.append("&")
          commande_robot.append("a")
          face="G"
  if face=="B":
    commande_robot.append("&")
  if face=="G":
    commande_robot.append("d")
  if face=="D":
    commande_robot.append("g")
  # print(commande_robot)
  return(commande_robot)



def commande_finale(commande_robot):
  """
  Cette fonction permet de transformer les instructions 'avancer fois n' en 'n fois avancer', cela permet une meilleur fluidité dans le mouvement du robot
  Pour cela on parcourt la liste, tant qu'on parcourt des "a" on ajoute 1 au compteur
  Lorsqu'on tombe sur gauche ou droite, on  remplit "instruction" avec le caractère correspondant au nombre de
  cases comptées et on ajoute ensuite gauche ou droite
  La suite de test en dehors de la boucle for sert à rajouter les dernieres instructions avancées puisque
  le remplissage dans la boucle ne se fait que si on tombe sur un g ou d
  """
  instruction=[]      #suite de caractères qui seront envoyés finalement au robot sans modification
  count=0
  for ins in commande_robot:
    if ins=="a":
      count+=1
    else :
      if count==1:                    #à chaque caractère correspond au commande qui indique au robot qu'il doit avancer de n cases.
        instruction.append("a")
      if count==2:
        instruction.append("b")
      if count==3:
        instruction.append("c")
      if count==4:
        instruction.append("e")
      if count==5:
        instruction.append("h")
      if count==6:
        instruction.append("i")
      if count==7:
        instruction.append("j")
      if count==8:
        instruction.append("k")
      if count==9:
        instruction.append("l")
      if count==10:
        instruction.append("m")
      if count==11:
        instruction.append("n")
      if count==12:
        instruction.append("o")
      if count==13:
        instruction.append("p")
      if count==14:
        instruction.append("q")
      if count==15:
        instruction.append("r")
      count=0
      instruction.append(ins)

  if count==1:
    instruction.append("a")
  if count==2:
    instruction.append("b")
  if count==3:
    instruction.append("c")
  if count==4:
    instruction.append("e")
  if count==5:
    instruction.append("h")
  if count==6:
    instruction.append("i")
  if count==7:
    instruction.append("j")
  if count==8:
    instruction.append("k")
  if count==9:
    instruction.append("l")
  if count==10:
    instruction.append("m")
  if count==11:
    instruction.append("n")
  if count==12:
    instruction.append("o")
  if count==13:
    instruction.append("p")
  if count==14:
    instruction.append("q")
  if count==15:
    instruction.append("r")

  # print(instruction)
  return(instruction)